
#ifndef _LINUX_SWSUSP_DEBUG_H
#define _LINUX_SWSUSP_DEBUG_H

/* Solely for comparison with the version specific patch revision */
#define SWSUSP_CORE_REVISION 0x201

#define SWSUSP_CORE_VERSION "2.0.0.90"
#define name_suspend "Software Suspend " SWSUSP_CORE_VERSION ": "

/* Same length to ensure one overwrites the other */
#define console_suspend " S U S P E N D   T O   D I S K " 
#define console_resume  "R E S U M E   F R O M   D I S K"

extern unsigned long suspend_action;
extern unsigned long suspend_result;
extern unsigned long suspend_debug_state;

#define TEST_RESULT_STATE(bit) (test_bit(bit, &suspend_result))
#define SET_RESULT_STATE(bit) (test_and_set_bit(bit, &suspend_result))
#define CLEAR_RESULT_STATE(bit) (test_and_clear_bit(bit, &suspend_result))

#define TEST_ACTION_STATE(bit) (test_bit(bit, &suspend_action))
#define SET_ACTION_STATE(bit) (test_and_set_bit(bit, &suspend_action))
#define CLEAR_ACTION_STATE(bit) (test_and_clear_bit(bit, &suspend_action))

#ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG
#define TEST_DEBUG_STATE(bit) (suspend_debug_state) && \
	(test_bit(bit, &suspend_debug_state))
#define SET_DEBUG_STATE(bit) (test_and_set_bit(bit, &suspend_debug_state))
#define CLEAR_DEBUG_STATE(bit) (test_and_clear_bit(bit, &suspend_debug_state))
#else
#define TEST_DEBUG_STATE(bit) (0)
#define SET_DEBUG_STATE(bit) (0)
#define CLEAR_DEBUG_STATE(bit) (0)
#endif

/* first status register - this is suspend's return code. */
#define SUSPEND_ABORTED			0
#define SUSPEND_ABORT_REQUESTED		1
#define SUSPEND_NOSTORAGE_AVAILABLE	2
#define SUSPEND_INSUFFICIENT_STORAGE	3
#define SUSPEND_FREEZING_FAILED		4
#define SUSPEND_UNEXPECTED_ALLOC	5
#define SUSPEND_KEPT_IMAGE		6
#define SUSPEND_WOULD_EAT_MEMORY	7
#define SUSPEND_UNABLE_TO_FREE_ENOUGH_MEMORY 8

/* second status register */
#define SUSPEND_REBOOT		0
#define SUSPEND_NO_OUTPUT	1
#define SUSPEND_PAUSE		2
#define SUSPEND_SLOW		3
#define SUSPEND_NOPAGESET2	7
#define SUSPEND_LOGALL		8
/* Set to disable compression when compiled in */
#define SUSPEND_NO_COMPRESSION	9
//#define SUSPEND_ENABLE_KDB	10
#define SUSPEND_CAN_CANCEL	11
#define SUSPEND_KEEP_IMAGE	13
#define SUSPEND_FREEZER_TEST	14
#define SUSPEND_FREEZER_TEST_SHOWALL 15
#define SUSPEND_SINGLESTEP	16
#define SUSPEND_PAUSE_NEAR_PAGESET_END 17

/* debug sections  - if debugging compiled in */
#define SUSPEND_ANY_SECTION	0
#define SUSPEND_FREEZER		1
#define SUSPEND_EAT_MEMORY 	2
#define SUSPEND_PAGESETS	3
#define SUSPEND_IO		4
#define SUSPEND_BMAP		5
#define SUSPEND_SWAP		9
#define SUSPEND_MEMORY		10
#define SUSPEND_RANGES		11
#define SUSPEND_SPINLOCKS	12
#define SUSPEND_MEM_POOL	13
#define SUSPEND_RANGE_PARANOIA	14
#define SUSPEND_NOSAVE		15
#define SUSPEND_INTEGRITY	16
/* debugging levels - if debugging compiled in.
 * These are straight integer values, not bit flags.
 * (Stored as LEVEL << 16 + section_flags).
 */
#define SUSPEND_ERROR		2
#define SUSPEND_LOW	 	3
#define SUSPEND_MEDIUM	 	4
#define SUSPEND_HIGH	  	5
#define SUSPEND_VERBOSE		6

/* 
 *   #defs that are dependant on CONFIG_SOFTWARE_SUSPEND_DEBUG
 *
 */
#if defined(CONFIG_SOFTWARE_SUSPEND_DEBUG)

#define NO_OUTPUT_OR_PAUSING (TEST_ACTION_STATE(SUSPEND_NO_OUTPUT))
#define CHECKMASK(mask) (((!mask) || (TEST_DEBUG_STATE(mask))) \
		&& (!NO_OUTPUT_OR_PAUSING))

void __printnolog(int restartline, const char *fmt, ...);
void __printlog(const char *fmt, ...);

#define printlog(mask, level, f, a...) do { \
	if (CHECKMASK(mask) && (console_loglevel >= level)) \
		__printlog(f, ##a); \
} while(0)

#define printnolog(mask, level, f, a...) do { \
	if (CHECKMASK(mask) && (console_loglevel >= level)) \
		__printnolog(f, ##a); \
} while(0)

#else // #ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG

#define printlog(...) do { } while(0)
#define printnolog(...) do { } while(0)
#define NO_OUTPUT_OR_PAUSING (TEST_ACTION_STATE(SUSPEND_NO_OUTPUT))
#define CHECKMASK(mask) (0)

#endif // #ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG

#endif
